'use strict';

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _includes2 = require('lodash/includes');

var _includes3 = _interopRequireDefault(_includes2);

var _map2 = require('lodash/map');

var _map3 = _interopRequireDefault(_map2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var program = require('commander');
var packageJson = require('../../package.json');
var path = require('path');


var defaultHost = process.platform === 'win32' ? 'localhost' : '0.0.0.0';

var directory = path.resolve('.');

program.version(packageJson.version).usage('[command] [options]');

program.command('develop').description('Start development server. Watches files and rebuilds and hot reloads if something changes') // eslint-disable-line max-len
.option('-H, --host <url>', 'Set host. Defaults to ' + defaultHost, defaultHost).option('-p, --port <port>', 'Set port. Defaults to 8000', '8000').option('-o, --open', 'Open the site in your browser for you.').action(function (command) {
  var develop = require('../utils/develop');
  var p = (0, _extends3.default)({}, command, {
    directory: directory
  });
  develop(p);
});

program.command('build').description('Build a Gatsby project.').option('--prefix-links', 'Build site with links prefixed (set prefix in your config).').action(function (command) {
  // Set NODE_ENV to 'production'
  process.env.NODE_ENV = 'production';

  var build = require('../utils/build');
  var p = (0, _extends3.default)({}, command, {
    directory: directory
  });
  build(p, function (err) {
    if (err) {
      throw err;
    } else {
      console.log('Done');
      process.exit();
    }
  });
});

program.command('serve-build').description('Serve built site.').option('-H, --host <url>', 'Set host. Defaults to ' + defaultHost, defaultHost).option('-p, --port <port>', 'Set port. Defaults to 8000', '8000').option('-o, --open', 'Open the site in your browser for you.').action(function (command) {
  var serve = require('../utils/serve-build');
  var p = (0, _extends3.default)({}, command, {
    directory: directory
  });
  serve(p);
});

program.command('new [rootPath] [starter]').description('Create new Gatsby project.').action(function (rootPath, starter) {
  var newCommand = require('../utils/new');
  newCommand(rootPath, starter);
});

program.on('--help', function () {
  console.log('To show subcommand help:\n\n    gatsby [command] -h\n');
});

// If the user types an unknown sub-command, just display the help.
var subCmd = process.argv.slice(2, 3)[0];
var cmds = (0, _map3.default)(program.commands, '_name');
cmds = cmds.concat(['--version', '-V']);

if (!(0, _includes3.default)(cmds, subCmd)) {
  program.help();
} else {
  program.parse(process.argv);
}