'use strict';

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _clone2 = require('lodash/clone');

var _clone3 = _interopRequireDefault(_clone2);

var _filter2 = require('lodash/filter');

var _filter3 = _interopRequireDefault(_filter2);

var _find2 = require('lodash/find');

var _find3 = _interopRequireDefault(_find2);

var _words2 = require('lodash/words');

var _words3 = _interopRequireDefault(_words2);

var _sortBy2 = require('lodash/sortBy');

var _sortBy3 = _interopRequireDefault(_sortBy2);

var _includes2 = require('lodash/includes');

var _includes3 = _interopRequireDefault(_includes2);

var _first2 = require('lodash/first');

var _first3 = _interopRequireDefault(_first2);

var _each2 = require('lodash/each');

var _each3 = _interopRequireDefault(_each2);

var _parseFilepath = require('parse-filepath');

var _parseFilepath2 = _interopRequireDefault(_parseFilepath);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _remark = require('remark');

var _remark2 = _interopRequireDefault(_remark);

var _sanitizeHtml = require('sanitize-html');

var _sanitizeHtml2 = _interopRequireDefault(_sanitizeHtml);

var _excerptHtml = require('excerpt-html');

var _excerptHtml2 = _interopRequireDefault(_excerptHtml);

var _remarkHtml = require('remark-html');

var _remarkHtml2 = _interopRequireDefault(_remarkHtml);

var _glob = require('glob');

var _glob2 = _interopRequireDefault(_glob);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _unistUtilSelect = require('unist-util-select');

var _unistUtilSelect2 = _interopRequireDefault(_unistUtilSelect);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _yamlJs = require('yaml-js');

var _yamlJs2 = _interopRequireDefault(_yamlJs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _graphql = require('graphql');

var _graphqlRelay = require('graphql-relay');

var _inferGraphqlType = require('./infer-graphql-type');

var _inferGraphqlType2 = _interopRequireDefault(_inferGraphqlType);

var _node = require('./node');

var _createPath = require('../bootstrap/create-path');

var _createPath2 = _interopRequireDefault(_createPath);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Setup Markdown parsing.
var remark = (0, _remark2.default)({ commonmark: true });
var thing = {};
(0, _remarkHtml2.default)(thing);

module.exports = function (directory) {
  return new _bluebird2.default(function (resolve, reject) {
    var pages = [];
    (0, _glob2.default)(directory + '/**/*.md', function (err, files) {
      if (err) return reject(err);
      (0, _each3.default)(files, function (file) {
        var page = {};
        page.src = _fs2.default.readFileSync(file, 'utf-8');
        var ast = remark.parse(page.src);
        page.ast = ast;
        page.bodyHTML = thing.Compiler.prototype.compile(page.ast);
        page.headings = (0, _unistUtilSelect2.default)(ast, 'heading').map(function (heading) {
          return {
            value: (0, _first3.default)((0, _unistUtilSelect2.default)(heading, 'text').map(function (text) {
              return text.value;
            })),
            depth: heading.depth
          };
        });
        var parsedFrontmatter = (0, _first3.default)((0, _unistUtilSelect2.default)(ast, 'yaml').map(function (heading) {
          return _yamlJs2.default.load(heading.value);
        }));
        var relativeDirectory = (0, _parseFilepath2.default)(_path2.default.relative(directory + '/pages/', file)).dirname;

        // Create path.
        var filePath = void 0;
        if ((0, _includes3.default)(relativeDirectory, '---')) {
          // This is for my blog bricolage.io. This will be moved out soonish.
          filePath = '/' + relativeDirectory.split('---')[1] + '/';
        } else {
          filePath = (0, _createPath2.default)(_path2.default.join(directory, 'pages'), file);
        }
        // TODO put linkPrefix in gatsby.config.js and somehow get certain
        // context stuff into gatsby-helpers.js
        // TODO post issue for why manifest paths wrong
        // TODO post stackoverflow about how to get static values — ping Joe.
        // Prefix stuff — just store this as another key "prefixedPath"?
        pages.push((0, _extends3.default)({
          id: _path2.default.relative(directory, _path2.default.resolve(file)),
          path: filePath
        }, page, {
          frontmatter: (0, _extends3.default)({
            path: filePath
          }, parsedFrontmatter)
        }));
      });
      pages = (0, _sortBy3.default)(pages, function (page) {
        return page.frontmatter.date;
      }).reverse();

      // TODO add numberOfWords field
      var markdownType = new _graphql.GraphQLObjectType({
        name: 'Markdown',
        fields: function fields() {
          return {
            id: {
              type: new _graphql.GraphQLNonNull(_graphql.GraphQLID)
            },
            path: {
              type: _graphql.GraphQLString
            },
            frontmatter: {
              type: new _graphql.GraphQLObjectType({
                name: 'Frontmatter',
                fields: frontmatterFields()
              })
            },
            headings: {
              type: new _graphql.GraphQLList(HeadingType),
              resolve: function resolve(page) {
                return page.headings;
              }
            },
            bodyMarkdown: {
              type: _graphql.GraphQLString,
              resolve: function resolve(page) {
                return page.src;
              }
            },
            bodyHTML: {
              type: _graphql.GraphQLString,
              resolve: function resolve(page) {
                return page.bodyHTML;
              }
            },
            timeToRead: {
              type: _graphql.GraphQLInt,
              resolve: function resolve(page) {
                var timeToRead = 0;
                var pureText = (0, _sanitizeHtml2.default)(page.bodyHTML, { allowTags: [] });
                var avgWPM = 265;
                var wordCount = (0, _words3.default)(pureText).length;
                timeToRead = Math.round(wordCount / avgWPM);
                if (timeToRead === 0) {
                  timeToRead = 1;
                }
                return timeToRead;
              }
            },
            excerpt: {
              type: _graphql.GraphQLString,
              args: {
                pruneLength: {
                  type: _graphql.GraphQLInt,
                  defaultValue: 140
                }
              },
              resolve: function resolve(page, _ref) {
                var pruneLength = _ref.pruneLength;

                return (0, _excerptHtml2.default)(page.bodyHTML, {
                  pruneLength: pruneLength
                });
              }
            },
            readNext: {
              type: _graphql.GraphQLString,
              resolve: function resolve(page) {
                return page.readNext;
              }
            }
          };
        },
        interfaces: [_node.nodeInterface]
      });

      var frontmatterFields = function frontmatterFields() {
        // Create object with the first example of every frontmatter key.
        var frontmatterFieldExamples = {};
        (0, _each3.default)(pages, function (page) {
          (0, _each3.default)(page.frontmatter, function (v, k) {
            if (!frontmatterFieldExamples[k]) {
              frontmatterFieldExamples[k] = v;
            }
          });
        });

        var fields = {};
        // TODO make this work with sub-objects.
        (0, _each3.default)(frontmatterFieldExamples, function (v, k) {
          var type = (0, _inferGraphqlType2.default)(k, v);
          if (type === 'DATE') {
            fields[k] = {
              type: _graphql.GraphQLString,
              args: {
                formatString: {
                  type: _graphql.GraphQLString
                }
              },
              resolve: function resolve(_ref2, _ref3) {
                var date = _ref2.date;
                var formatString = _ref3.formatString;

                if (formatString) {
                  return (0, _moment2.default)(date).format(formatString);
                } else {
                  return date;
                }
              }
            };
          } else {
            fields[k] = { type: (0, _inferGraphqlType2.default)(k, v) };
          }
        });

        if (fields.readNext) {
          fields.readNext = {
            type: markdownType,
            resolve: function resolve(frontmatter) {
              return (0, _find3.default)(pages, function (page) {
                return page.path === frontmatter.readNext;
              });
            }
          };
        }

        return fields;
      };

      var HeadingType = new _graphql.GraphQLObjectType({
        name: 'Heading',
        fields: {
          value: {
            type: _graphql.GraphQLString,
            resolve: function resolve(heading) {
              return heading.value;
            }
          },
          depth: {
            type: _graphql.GraphQLInt,
            resolve: function resolve(heading) {
              return heading.depth;
            }
          }
        }
      });

      var _connectionDefinition = (0, _graphqlRelay.connectionDefinitions)({
        nodeType: markdownType,
        connectionFields: function connectionFields() {
          return {
            totalCount: {
              type: _graphql.GraphQLInt
            }
          };
        }
      }),
          markdownConnection = _connectionDefinition.connectionType;

      var markdownTypes = {
        allMarkdown: {
          type: markdownConnection,
          description: 'All markdown files',
          args: (0, _extends3.default)({}, _graphqlRelay.connectionArgs, {
            tag: {
              type: _graphql.GraphQLString,
              description: 'Filter by posts that have this tag'
            }
          }),
          resolve: function resolve(markdown, args) {
            var filteredPages = void 0;
            // Specifc for my blog bricolage.io. Will be adding
            // general filtering feature soonish.
            if (args.tag) {
              filteredPages = (0, _filter3.default)(pages, function (page) {
                if (!page.frontmatter.tags) {
                  return false;
                } else {
                  return (0, _includes3.default)(page.frontmatter.tags.map(function (tag) {
                    return tag.toLowerCase();
                  }), args.tag.toLowerCase());
                }
              });
            } else {
              filteredPages = (0, _clone3.default)(pages);
            }
            var result = (0, _graphqlRelay.connectionFromArray)(filteredPages, args);
            result.totalCount = filteredPages.length;
            return result;
          }
        },
        markdown: {
          type: markdownType,
          args: {
            path: {
              type: new _graphql.GraphQLNonNull(_graphql.GraphQLString),
              description: 'URL path to markdown page'
            }
          },
          resolve: function resolve(root, args) {
            return (0, _find3.default)(pages, function (page) {
              return page.path === args.path;
            });
          }
        }
      };
      return resolve(markdownTypes);
    });
  });
};