'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _isFunction2 = require('lodash/isFunction');

var _isFunction3 = _interopRequireDefault(_isFunction2);

var _isObject2 = require('lodash/isObject');

var _isObject3 = _interopRequireDefault(_isObject2);

var _includes2 = require('lodash/includes');

var _includes3 = _interopRequireDefault(_includes2);

exports.default = ValidateWebpackConfig;

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _webpackValidator = require('webpack-validator');

var _webpackValidator2 = _interopRequireDefault(_webpackValidator);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var modifyWebpackConfig = void 0;
try {
  var gatsbyNodeConfig = _path2.default.resolve(process.cwd(), './gatsby-node');
  var nodeConfig = require(gatsbyNodeConfig);
  modifyWebpackConfig = nodeConfig.modifyWebpackConfig;
} catch (e) {
  if (e.code !== 'MODULE_NOT_FOUND' && !(0, _includes3.default)(e.Error, 'gatsby-node')) {
    console.log(e);
  }
}

function ValidateWebpackConfig(module, config, stage) {
  var userWebpackConfig = module(config);
  if (modifyWebpackConfig) {
    userWebpackConfig = modifyWebpackConfig(userWebpackConfig, stage);

    (0, _invariant2.default)((0, _isObject3.default)(userWebpackConfig) && (0, _isFunction3.default)(userWebpackConfig.resolve), '\n      You must return an webpack-configurator instance when modifying the Webpack config.\n      Returned: ' + userWebpackConfig + '\n      stage: ' + stage + '\n      ');
  }

  var validationState = (0, _webpackValidator2.default)(userWebpackConfig.resolve(), {
    returnValidation: true
  });

  if (!validationState.error) {
    return userWebpackConfig;
  }

  console.log('There were errors with your webpack config:');
  validationState.error.details.forEach(function (err, index) {
    console.log('[' + (index + 1) + ']');
    console.log(err.path);
    console.log(err.type, ',', err.message);
    console.log('\n');
  });

  return process.exit(1);
}
module.exports = exports['default'];