'use strict';

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _from = require('babel-runtime/core-js/array/from');

var _from2 = _interopRequireDefault(_from);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _uniq2 = require('lodash/uniq');

var _uniq3 = _interopRequireDefault(_uniq2);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _webpackConfigurator = require('webpack-configurator');

var _webpackConfigurator2 = _interopRequireDefault(_webpackConfigurator);

var _extractTextWebpackPlugin = require('extract-text-webpack-plugin');

var _extractTextWebpackPlugin2 = _interopRequireDefault(_extractTextWebpackPlugin);

var _staticSiteGeneratorWebpackPlugin = require('static-site-generator-webpack-plugin');

var _staticSiteGeneratorWebpackPlugin2 = _interopRequireDefault(_staticSiteGeneratorWebpackPlugin);

var _webpackStatsPlugin = require('webpack-stats-plugin');

var _webpackModifyValidate = require('./webpack-modify-validate');

var _webpackModifyValidate2 = _interopRequireDefault(_webpackModifyValidate);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var debug = require('debug')('gatsby:webpack-config');
//import HardSourceWebpackPlugin from 'hard-source-webpack-plugin'

var WebpackMD5Hash = require('webpack-md5-hash');
var OfflinePlugin = require('offline-plugin');
var ChunkManifestPlugin = require('chunk-manifest-webpack-plugin');

var _require = require('../utils/globals'),
    pagesDB = _require.pagesDB,
    siteDB = _require.siteDB;

var _require2 = require('./js-chunk-names'),
    layoutComponentChunkName = _require2.layoutComponentChunkName;

var babelConfig = require('./babel-config');

// Five stages or modes:
//   1) develop: for `gatsby develop` command, hot reload and CSS injection into page
//   2) develop-html: same as develop without react-hmre in the babel config for html renderer
//   3) build-css: build styles.css file
//   4) build-html: build all HTML files
//   5) build-javascript: Build js chunks for Single Page App in production

module.exports = function (program, directory, suppliedStage) {
  var webpackPort = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 1500;
  var pages = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : [];

  var babelStage = suppliedStage;
  var stage = suppliedStage === 'develop-html' ? 'develop' : suppliedStage;

  debug('Loading webpack config for stage "' + stage + '"');
  function output() {
    switch (stage) {
      case 'develop':
        return {
          path: directory,
          filename: '[name].js',
          publicPath: 'http://' + program.host + ':' + webpackPort + '/'
        };
      case 'build-css':
        // Webpack will always generate a resultant javascript file.
        // But we don't want it for this step. Deleted by build-css.js.
        return {
          path: directory + '/public',
          filename: 'bundle-for-css.js',
          publicPath: program.prefixLinks ? siteDB().get('config').linkPrefix + '/' : '/'
        };
      case 'build-html':
        // A temp file required by static-site-generator-plugin. See plugins() below.
        // Deleted by build-html.js, since it's not needed for production.
        return {
          path: directory + '/public',
          filename: 'render-page.js',
          libraryTarget: 'umd',
          publicPath: program.prefixLinks ? siteDB().get('config').linkPrefix + '/' : '/'
        };
      case 'build-javascript':
        return {
          //filename: '[name].js',
          filename: '[name]-[chunkhash].js',
          chunkFilename: '[name]-[chunkhash].js',
          path: directory + '/public',
          publicPath: program.prefixLinks ? siteDB().get('config').linkPrefix + '/' : '/'
        };
      default:
        throw new Error('The state requested ' + stage + ' doesn\'t exist.');
    }
  }

  function entry() {
    switch (stage) {
      case 'develop':
        return {
          commons: [require.resolve('webpack-dev-server/client') + '?http://' + program.host + ':' + webpackPort + '/', require.resolve('webpack/hot/only-dev-server'), require.resolve('react-hot-loader/patch'), directory + '/.intermediate-representation/app']
        };
      case 'build-css':
        return {
          main: directory + '/.intermediate-representation/app'
        };
      case 'build-html':
        return {
          main: __dirname + '/static-entry'
        };
      case 'build-javascript':
        return {
          app: directory + '/.intermediate-representation/production-app'
        };
      default:
        throw new Error('The state requested ' + stage + ' doesn\'t exist.');
    }
  }

  function plugins() {
    switch (stage) {
      case 'develop':
        return [new _webpack2.default.optimize.OccurenceOrderPlugin(), new _webpack2.default.HotModuleReplacementPlugin(), new _webpack2.default.NoErrorsPlugin(), new _webpack2.default.DefinePlugin({
          'process.env': {
            NODE_ENV: (0, _stringify2.default)(process.env.NODE_ENV ? process.env.NODE_ENV : 'development')
          },
          __PREFIX_LINKS__: program.prefixLinks,
          __LINK_PREFIX__: (0, _stringify2.default)(siteDB().get('config').linkPrefix)
        }),
        // Names module ids with their filepath. We use this in development
        // to make it easier to see what modules have hot reloaded, etc. as
        // the numerical IDs aren't useful. In production we use numerical module
        // ids to reduce filesize.
        new _webpack2.default.NamedModulesPlugin()];
      case 'build-css':
        return [new _webpack2.default.DefinePlugin({
          'process.env': {
            NODE_ENV: (0, _stringify2.default)(process.env.NODE_ENV ? process.env.NODE_ENV : 'production')
          },
          __PREFIX_LINKS__: program.prefixLinks,
          __LINK_PREFIX__: (0, _stringify2.default)(siteDB().get('config').linkPrefix)
        }), new _extractTextWebpackPlugin2.default('styles.css', { allChunks: true })];
      case 'build-html':
        return [new _staticSiteGeneratorWebpackPlugin2.default('render-page.js', pages), new _webpack2.default.DefinePlugin({
          'process.env': {
            NODE_ENV: (0, _stringify2.default)(process.env.NODE_ENV ? process.env.NODE_ENV : 'production')
          },
          __PREFIX_LINKS__: program.prefixLinks,
          __LINK_PREFIX__: (0, _stringify2.default)(siteDB().get('config').linkPrefix)
        }), new _extractTextWebpackPlugin2.default('build-html-styles.css')];
      case 'build-javascript':
        {
          // Get array of page template component names.
          var components = (0, _from2.default)(pagesDB().values()).map(function (page) {
            return page.component;
          });
          components = components.map(function (component) {
            return layoutComponentChunkName(program.directory, component);
          });
          components = (0, _uniq3.default)(components);
          return [
          // Moment.js includes 100s of KBs of extra localization data
          // by default in Webpack that most sites don't want.
          // This line disables that.
          // TODO remove this now that loading moment.js isn't common w/ new
          // graphql data layer?
          new _webpack2.default.IgnorePlugin(/^\.\/locale$/, /moment$/), new WebpackMD5Hash(), new _webpack2.default.optimize.DedupePlugin(),
          // Extract "commons" chunk from the app entry and all
          // page components.
          new _webpack2.default.optimize.CommonsChunkPlugin({
            name: 'commons',
            chunks: ['app'].concat((0, _toConsumableArray3.default)(components)),
            // The more page components there are, the higher we raise the bar
            // for merging in page-specific JS libs into the commons chunk. The
            // two principles here is a) keep the TTI (time to interaction) as
            // low as possible so that means keeping commons.js small with
            // critical code (e.g. React) and b) is we want to push JS
            // parse/eval work as close as possible to when it's used.  Since
            // most people don't navigate to most pages, take tradeoff of
            // loading/evaling modules multiple times over loading/evaling lots
            // of unused code on the initial opening of the app.
            //
            // Use Math.max as there must be at least two chunks.
            minChunks: Math.max(2, Math.floor(components.length / 3))
          }),
          // Add a few global variables. Set NODE_ENV to production (enables
          // optimizations for React) and whether prefixing links is enabled
          // (__PREFIX_LINKS__) and what the link prefix is (__LINK_PREFIX__).
          new _webpack2.default.DefinePlugin({
            'process.env': {
              NODE_ENV: (0, _stringify2.default)(process.env.NODE_ENV ? process.env.NODE_ENV : 'production')
            },
            __PREFIX_LINKS__: program.prefixLinks,
            __LINK_PREFIX__: (0, _stringify2.default)(siteDB().get('config').linkPrefix)
          }),
          // Extract CSS so it doesn't get added to JS bundles.
          new _extractTextWebpackPlugin2.default('build-js-styles.css'),
          // Write out mapping between chunk names and their hashed names. We use
          // this to add the needed javascript files to each HTML page.
          new _webpackStatsPlugin.StatsWriterPlugin(),
          // Extract the webpack chunk manifest out of commons.js so commons.js
          // doesn't get changed everytime you build. This increases the cache-hit
          // rate for commons.js.
          new ChunkManifestPlugin({
            filename: "chunk-manifest.json",
            manifestVariable: "webpackManifest"
          }),
          // Minify Javascript.
          new _webpack2.default.optimize.UglifyJsPlugin({
            compress: {
              screw_ie8: true, // React doesn't support IE8
              warnings: false
            },
            mangle: {
              screw_ie8: true
            },
            output: {
              comments: false,
              screw_ie8: true
            }
          }),
          // Ensure module order stays the same. Supposibly fixed in webpack 2.0.
          new _webpack2.default.optimize.OccurenceOrderPlugin(),
          // Enable the offline plugin to add a service worker.
          new OfflinePlugin({
            AppCache: false,
            publicPath: program.prefixLinks ? siteDB().get('config').linkPrefix + '/' : '/',
            relativePaths: false,
            ServiceWorker: {
              events: true
            }
          })];
        }
      default:
        throw new Error('The state requested ' + stage + ' doesn\'t exist.');
    }
  }

  function resolve() {
    return {
      extensions: ['', '.js', '.jsx', '.cjsx', '.coffee'],
      // Hierarchy of directories for Webpack to look for module.
      // First is the site directory.
      // Then in the special directory of isomorphic modules Gatsby ships with.
      root: [directory, _path2.default.resolve(__dirname, '..', 'isomorphic')],
      modulesDirectories: [directory + '/node_modules', 'node_modules']
    };
  }

  function devtool() {
    switch (stage) {
      case 'develop':
        return 'eval';
      case 'build-html':
        return false;
      case 'build-javascript':
        return 'source-map';
      default:
        return false;
    }
  }

  function module(config) {
    // Common config for every env.
    config.loader('cjsx', {
      test: /\.cjsx$/,
      loaders: ['coffee', 'cjsx']
    });
    config.loader('js', {
      test: /\.jsx?$/, // Accept either .js or .jsx files.
      exclude: /(node_modules|bower_components)/,
      loader: 'babel',
      query: babelConfig(program, babelStage)
    });
    config.loader('coffee', {
      test: /\.coffee$/,
      loader: 'coffee'
    });
    config.loader('json', {
      test: /\.json$/,
      loaders: ['json']
    });
    // "file" loader makes sure those assets end up in the `public` folder.
    // When you `import` an asset, you get its filename.
    config.loader('file-loader', {
      test: /\.(ico|eot|otf|webp|ttf)(\?.*)?$/,
      loader: 'file',
      query: {
        name: 'static/[name].[hash:8].[ext]'
      }
    });
    // "url" loader works just like "file" loader but it also embeds
    // assets smaller than specified size as data URLs to avoid requests.
    config.loader('url-loader', {
      test: /\.(svg|jpg|jpeg|png|gif|mp4|webm|wav|mp3|m4a|aac|oga)(\?.*)?$/,
      loader: 'url',
      query: {
        limit: 7500,
        name: 'static/[name].[hash:8].[ext]'
      }
    });
    // Font loader.
    config.loader('woff', {
      test: /\.woff(2)?(\?v=[0-9]\.[0-9]\.[0-9])?$/,
      loader: 'url',
      query: {
        limit: 15000, // Set high limit for inlining fonts as they're in the
        // critical path for rendering.
        name: 'static/[name].[hash:8].[ext]',
        mimetype: 'application/font-woff'
      }
    });

    var cssModulesConf = 'css?modules&minimize&importLoaders=1';
    var cssModulesConfDev = cssModulesConf + '&sourceMap&localIdentName=[name]---[local]---[hash:base64:5]';

    switch (stage) {
      case 'develop':

        config.loader('css', {
          test: /\.css$/,
          exclude: /\.module\.css$/,
          loaders: ['style', 'css', 'postcss']
        });

        // CSS modules
        config.loader('cssModules', {
          test: /\.module\.css$/,
          loaders: ['style', cssModulesConfDev, 'postcss']
        });

        config.merge({
          postcss: function postcss(wp) {
            return [require('postcss-import')({ addDependencyTo: wp }), require('postcss-cssnext')({ browsers: 'last 2 versions' }), require('postcss-browser-reporter'), require('postcss-reporter')];
          }
        });
        return config;

      case 'build-css':
        config.loader('css', {
          test: /\.css$/,
          loader: _extractTextWebpackPlugin2.default.extract(['css?minimize', 'postcss']),
          exclude: /\.module\.css$/
        });

        // CSS modules
        config.loader('cssModules', {
          test: /\.module\.css$/,
          loader: _extractTextWebpackPlugin2.default.extract('style', [cssModulesConf, 'postcss'])
        });
        config.merge({
          postcss: [require('postcss-import')(), require('postcss-cssnext')({
            browsers: 'last 2 versions'
          })]
        });
        return config;

      case 'build-html':
        // We don't deal with CSS at all when building the HTML.
        // The 'null' loader is used to prevent 'module not found' errors.
        // On the other hand CSS modules loaders are necessary.

        config.loader('css', {
          test: /\.css$/,
          exclude: /\.module\.css$/,
          loader: 'null'
        });

        // CSS modules
        config.loader('cssModules', {
          test: /\.module\.css$/,
          loader: _extractTextWebpackPlugin2.default.extract('style', [cssModulesConf, 'postcss'])
        });

        return config;

      case 'build-javascript':
        // we don't deal with css at all when building the javascript.  but
        // still need to process the css so offline-plugin knows about the
        // various assets referenced in your css.
        //
        // It's also necessary to process CSS Modules so your JS knows the
        // classNames to use.

        config.loader('css', {
          test: /\.css$/,
          exclude: /\.module\.css$/,
          //loader: `null`,
          loader: _extractTextWebpackPlugin2.default.extract(['css'])
        });

        // CSS modules
        config.loader('cssModules', {
          test: /\.module\.css$/,
          loader: _extractTextWebpackPlugin2.default.extract('style', [cssModulesConf, 'postcss'])
        });

        return config;

      default:
        return config;
    }
  }

  function resolveLoader() {
    var root = [_path2.default.resolve(directory, 'node_modules')];

    var userLoaderDirectoryPath = _path2.default.resolve(directory, 'loaders');

    try {
      if (_fs2.default.statSync(userLoaderDirectoryPath).isDirectory()) {
        root.push(userLoaderDirectoryPath);
      }
    } catch (e) {
      if (e && e.code !== 'ENOENT') {
        console.log(e);
      }
    }

    return {
      root: root,
      modulesDirectories: ['node_modules']
    };
  }

  var config = new _webpackConfigurator2.default();

  config.merge({
    // Context is the base directory for resolving the entry option.
    context: '' + directory,
    node: {
      __filename: true
    },
    entry: entry(),
    debug: true,
    // Certain "isomorphic" packages have different entry points for browser and server
    // (see https://github.com/defunctzombie/package-browser-field-spec);
    // setting the target tells webpack which file to include, ie. browser vs main.
    target: stage === 'build-html' ? 'node' : 'web',
    profile: stage === 'production',
    devtool: devtool(),
    output: output(),
    resolveLoader: resolveLoader(),
    plugins: plugins(),
    resolve: resolve(),
    recordsPath: process.cwd() + '/.webpack-records/records-' + stage + '.json'
  });

  return (0, _webpackModifyValidate2.default)(module, config, stage);
};